/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.ftp.internal;

import java.io.*;
import java.net.Socket;
import java.net.SocketException;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.ftp.FtpException;
import org.eclipse.ftp.internal.streams.*;
import org.eclipse.ftp.internal.streams.PollingInputStream;
import org.eclipse.ftp.internal.streams.PollingOutputStream;

public abstract class DataConnection implements Runnable {
	private IProgressMonitor monitor;

	private Socket socket;
	private InputStream socketIn;
	private OutputStream socketOut;
	private FtpException pendingException;
	private Thread thread;
	private boolean isReady;
	
	protected DataConnection() {
		this.isReady = false;
	}
	
	/**
	 * Starts a background thread to open the socket, returns immediately.
	 * The progress monitor is passed to the worker if successful.
	 * @param monitor the progress monitor
	 * @throws FtpException
	 */
	public void open(IProgressMonitor monitor) throws FtpException {
		this.monitor = FtpPlugin.monitorFor(monitor);
		thread = new Thread(this, FtpPlugin.getResourceString("FTPClient.FTPDataTransferWorker")); //$NON-NLS-1$
		thread.start();
	}
	
	/**
	 * Stops the background thread and closes the socket, optionally blocks until completed.
	 * Uses the progress monitor specified in open after the worker exits.
	 * @param interrupt if true, interrupts the operation in progress, eats any exceptions and
	 *        returns immediately, else blocks until the operation completes
	 */
	public void close(boolean interrupt) throws FtpException {
		if (interrupt) {
			thread.interrupt();
		}
		if (pendingException != null) throw pendingException;
	}
	
	/**
	 * Checks the status of the receiver and yields to other threads if 
	 * nothing of interest is happened yet
	 */
	public boolean isReady() throws FtpException {
		// Throw any exceptions that happened while trying to connect
		if (pendingException != null) throw pendingException;
		if (! isReady) {
			Thread.yield();
		}
		return isReady;
	}

	/**
	 * Private.
	 */
	public void run() {
		isReady = false;
		if (Thread.interrupted()) return;
		try {
			try {
				socket = openSocket();
				if (Thread.interrupted()) return;
				try {
					socket.setKeepAlive(false);
				} catch (SocketException e) {
					throw new FtpCommunicationException(FtpPlugin.getResourceString("FTPClient.CannotSetSocketOptions"), e); //$NON-NLS-1$
				}
				if (Thread.interrupted()) return;
				isReady = true;
			} finally {
				if (!isReady) {
					closeSocket();
				}
			}
		} catch (FtpException e) {
			pendingException = e;
		}
	}

	protected abstract Socket openSocket() throws FtpException;

	public void closeSocket() throws FtpException {
		if (socket == null) return;
		// ensure the streams and the socket are closed properly even if multiple exceptions occur
		try {
			try {
				try {
					if (socketIn != null) socketIn.close();
				} finally {
					socketIn = null;
					if (socketOut != null) socketOut.close();
				}
			} finally {
				socketOut = null;
				socket.close();
			}
		} catch (IOException e) {
			throw new FtpCommunicationException(FtpPlugin.getResourceString("FTPClient.ErrorDisconnecting"), e); //$NON-NLS-1$
		} finally {
			socket = null;
		}
	}
	
	public InputStream getSocketInputStream() throws FtpException {
		try {
			socket.shutdownOutput();
			socket.setSoTimeout(1000);
			socket.setReceiveBufferSize(FtpPlugin.getDefault().getReceiveBufferSize());
			socketIn = new PollingInputStream(socket.getInputStream(),
				FtpPlugin.getDefault().getTimeout(), monitor);
			return socketIn;
		} catch (IOException e) {
			throw new FtpCommunicationException(FtpPlugin.getResourceString("DataConnection.CannotOpenInput"), e); //$NON-NLS-1$
		}
	}
	
	public OutputStream getSocketOutputStream() throws FtpException {
		try {
			socket.shutdownInput();
			socket.setSendBufferSize(FtpPlugin.getDefault().getSendBufferSize());
			socketOut = new PollingOutputStream(new TimeoutOutputStream(socket.getOutputStream(),
				8192 /*bufferSize*/, 1000 /*writeTimeout*/, 1000 /*closeTimeout*/),
				FtpPlugin.getDefault().getTimeout(), monitor);
			return socketOut;
		}catch (IOException e) {
			throw new FtpCommunicationException(FtpPlugin.getResourceString("DataConnection.CannotOpenInput"), e); //$NON-NLS-1$
		}
	}
	
	public InputStream getDataTransferInputStream() throws FtpException {
		InputStream in =  new DataTransferInputStream(this);
		return in;
	}
}
