/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.target.internal.ide;

import java.io.File;
import java.io.InputStream;
import java.util.Date;

import org.eclipse.core.resources.*;
import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.*;
import org.eclipse.jface.util.Assert;
import org.eclipse.target.*;

/**
 * A target resource that wraps an IResource
 */
public class IdeTargetResource extends TargetResource {

	private IResource resource;
	
	/**
	 * Create a target resource that wraps the given IResource
	 * @param site the IResource site
	 * @param resource the workspace resource
	 * @return
	 */
	public static ITargetResource createResource(Site site, IResource resource) {
		return new IdeTargetResource(site, resource);
	}
	
	private IdeTargetResource(Site site, IResource resource) {
		super(site);
		this.resource = resource;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.target.TargetResource#createResource(java.lang.String, boolean)
	 */
	protected ITargetResource createResource(String relativePath, boolean isDirectory) {
		Assert.isTrue(isDirectory());
		IResource child;
		if (isDirectory) {
			child = ((IContainer)resource).getFolder(new Path(relativePath));
		} else {
			child = ((IContainer)resource).getFile(new Path(relativePath));
		}
		return createResource(getSite(), child);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.target.ITargetResource#getRelativePath()
	 */
	public String getRelativePath() {
		return resource.getFullPath().toString();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.target.ITargetResource#isDirectory()
	 */
	public boolean isDirectory() {
		return resource.getType() != IResource.FILE;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.target.ITargetResource#exists(org.eclipse.core.runtime.IProgressMonitor)
	 */
	public boolean exists(IProgressMonitor monitor) throws CoreException {
		return resource.exists();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.target.ITargetResource#members(org.eclipse.core.runtime.IProgressMonitor)
	 */
	public ITargetResource[] members(IProgressMonitor monitor) throws CoreException {
		if (isDirectory()) {
			IResource[] members = ((IContainer)resource).members();
			ITargetResource[] result = new ITargetResource[members.length];
			for (int i = 0; i < members.length; i++) {
				IResource resource = members[i];
				result[i] = createResource(getSite(), resource);
			}
			return result;
		} else {
			return new ITargetResource[0];
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.target.ITargetResource#getContents(org.eclipse.core.runtime.IProgressMonitor)
	 */
	public InputStream getContents(IProgressMonitor progress) throws CoreException {
		if (isDirectory()) 
			return null;
		return ((IFile)resource).getContents();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.target.ITargetResource#setContents(java.io.InputStream, long, org.eclipse.core.runtime.IProgressMonitor)
	 */
	public void setContents(InputStream stream, long size, IProgressMonitor progress) throws CoreException {
		Assert.isTrue(!isDirectory());
		IFile file = (IFile)resource;
		if (file.exists()) {
			file.setContents(stream, false, true, progress);
		} else {
			file.create(stream, false, progress);
		}

	}

	/* (non-Javadoc)
	 * @see org.eclipse.target.ITargetResource#delete(org.eclipse.core.runtime.IProgressMonitor)
	 */
	public void delete(IProgressMonitor monitor) throws CoreException {
		// TODO: Speced to not be deep
		resource.delete(false, monitor);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.target.ITargetResource#fetchProperties(org.eclipse.core.runtime.QualifiedName[], boolean, org.eclipse.core.runtime.IProgressMonitor)
	 */
	public QualifiedName[] fetchProperties(QualifiedName[] properties, boolean refresh, IProgressMonitor monitor) throws CoreException {
		return getSupportedProperties();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.target.ITargetResource#isAvailable(org.eclipse.core.runtime.QualifiedName)
	 */
	public boolean isAvailable(QualifiedName property) {
		return isSupported(property);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.target.ITargetResource#getProperty(org.eclipse.core.runtime.QualifiedName)
	 */
	public String getProperty(QualifiedName property) throws CoreException {
		if (property.equals(SIZE_PROPERTY)) {
			return Long.toString(getSize());
		}
		if (property.equals(LAST_MODIFIED_TIMESTAMP_PROPERTY)) {
			return new Date(resource.getLocalTimeStamp()).toString();
		}
		if (property.equals(CONTENT_IDENTIFIER_PROPERTY)) {
			return new Date(resource.getLocalTimeStamp()).toString() + getSize();
		}
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.target.ITargetResource#getSize()
	 */
	public long getSize() {
		if (isDirectory())
			return 0;
		File file = getIOFile();
		if (file == null) {
			return 0;
		}
		return file.length();
	}

	private File getIOFile() {
		IPath location = resource.getLocation();
		if(location!=null) {
			return location.toFile();
		}
		return null;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.target.ITargetResource#mkdir(org.eclipse.core.runtime.IProgressMonitor)
	 */
	public void mkdir(IProgressMonitor monitor) throws CoreException {
		Assert.isTrue(isDirectory());
		if (resource.getType() == IResource.FOLDER) {
			((IFolder)resource).create(false, true, monitor);
		} else if (resource.getType() == IResource.PROJECT) {
			IProject project = (IProject)resource;
			monitor.beginTask(null, 100);
			project.create(Policy.subMonitorFor(monitor, 50));
			project.open(Policy.subMonitorFor(monitor, 50));
			monitor.done();
		}

	}

	/* (non-Javadoc)
	 * @see org.eclipse.target.ITargetResource#mkdirs(org.eclipse.core.runtime.IProgressMonitor)
	 */
	public void mkdirs(IProgressMonitor monitor) throws CoreException {
		if (resource.getType() == IResource.ROOT) return;
		IContainer parent = resource.getParent();
		if (!parent.exists()) {
			createResource(getSite(), parent).mkdirs(monitor);
		}
		mkdir(monitor);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	 */
	public Object getAdapter(Class adapter) {
		if (adapter.equals(IResource.class)) {
			return resource;
		}
		return super.getAdapter(adapter);
	}

	/**
	 * Return the underlying IResource.
	 * @return the underlying IResource.
	 */
	public IResource getIResource() {
		return resource;
	}

}
